"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildScript = void 0;
const fs_1 = __importDefault(require("fs"));
const typescript_parser_1 = require("typescript-parser");
const utils_1 = require("./utils");
const defaultTypes_1 = require("./defaultTypes");
function buildScript(srcDir, outputPath, isStrictMode) {
    return __awaiter(this, void 0, void 0, function* () {
        let output = "";
        const fileNames = [];
        (0, utils_1.readAllFiles)(srcDir, fileNames, (fileName) => fileName.endsWith(".ts"));
        const parser = new typescript_parser_1.TypescriptParser();
        const parsed = yield parser.parseFiles(fileNames, srcDir);
        const arraySanitizers = [];
        for (const file of parsed) {
            for (const declaration of file.declarations) {
                if (declaration instanceof typescript_parser_1.InterfaceDeclaration) {
                    output += `function sanitize${declaration.name}(checker: any) {\n`;
                    if (declaration.properties.length > 0) {
                        if (!isStrictMode) {
                            for (const property of declaration.properties) {
                                if (property.type == "number") {
                                    output += `if (typeof checker.${property.name} == "string" && Boolean(checker.${property.name}.trim()) && !Number.isNaN(Number(checker.${property.name}))) {\n`;
                                    output += `checker.${property.name} = Number(checker.${property.name})\n`;
                                    output += `}\n\n`;
                                }
                                if (property.type == "boolean") {
                                    output += `if (typeof checker.${property.name} == "string" && (checker.${property.name} == "true" || checker.${property.name} == "false")) {\n`;
                                    output += `checker.${property.name} = checker.${property.name} == "true"\n`;
                                    output += `}\n\n`;
                                }
                            }
                        }
                        output += `if (`;
                        const ifStatements = [];
                        for (const property of declaration.properties) {
                            property.type = property.type || "any";
                            const isArrayType = property.type.includes("[]");
                            let arrayDimension = 0;
                            if (isArrayType) {
                                while (property.type.includes("[]")) {
                                    if (!arraySanitizers.includes(property.type)) {
                                        arraySanitizers.push(property.type);
                                    }
                                    property.type = property.type.replace("[]", "");
                                    arrayDimension++;
                                }
                            }
                            if (property.type == "any" ||
                                property.type == "unknown") {
                                continue;
                            }
                            let statement = "";
                            if (isArrayType) {
                                let santizerName = `sanitize${property.type}`;
                                for (let i = 0; i < arrayDimension; i++) {
                                    santizerName += "Array";
                                }
                                statement = `!${santizerName}(checker.${property.name})`;
                            }
                            else if (defaultTypes_1.defaultTypes.includes(property.type)) {
                                statement = `typeof checker.${property.name} != "${property.type}"`;
                            }
                            else {
                                statement = `!sanitize${property.type}(checker.${property.name})`;
                            }
                            if (property.isOptional) {
                                statement = `(checker.${property.name} != undefined && ${statement})`;
                            }
                            ifStatements.push(statement);
                        }
                        output += ifStatements.join(" || ");
                        output += `) {\nreturn false;\n}\n`;
                    }
                    output += "return true;\n}\n\n";
                }
            }
        }
        for (const arraySanitizer of arraySanitizers) {
            let originType = arraySanitizer;
            let arrayDimension = 0;
            while (originType.includes("[]")) {
                originType = originType.replace("[]", "");
                arrayDimension++;
            }
            let santizerName = `sanitize${originType}`;
            for (let i = 0; i < arrayDimension; i++) {
                santizerName += "Array";
            }
            let subSantizerName = `${originType}`;
            for (let i = 0; i < arrayDimension - 1; i++) {
                subSantizerName += "Array";
            }
            output += `function ${santizerName}(checker: any) {\n`;
            output += `if (!Array.isArray(checker)) {\nreturn false;\n}\n\n`;
            output += `for (let i = 0; i < checker.length; i++) {\n`;
            output += `if (`;
            const ifStatements = [];
            if (defaultTypes_1.defaultTypes.includes(subSantizerName)) {
                ifStatements.push(`typeof checker[i] != "${subSantizerName}"`);
            }
            else {
                ifStatements.push(`!sanitize${subSantizerName}(checker[i])`);
            }
            output += ifStatements.join(" || ");
            output += `) {\nreturn false;\n}\n`;
            output += `}\n`;
            output += `return true;\n}\n\n`;
        }
        fs_1.default.writeFileSync(outputPath, output);
    });
}
exports.buildScript = buildScript;
